# JFrog Distribution Helm Chart

## Prerequisites Details

* Kubernetes 1.8+

## Chart Details
This chart does the following:

* Deploy PostgreSQL database.
* Deploy Redis.
* Deploy distributor.
* Deploy distribution.

## Requirements
- A running Kubernetes cluster
- Dynamic storage provisioning enabled
- Default StorageClass set to allow services using the default StorageClass for persistent storage
- A running Artifactory Enterprise Plus
- [Kubectl](https://kubernetes.io/docs/tasks/tools/install-kubectl/) installed and setup to use the cluster
- [Helm](https://helm.sh/) installed and setup to use the cluster (helm init)

## Installing the Chart

### Add JFrog Helm repository
Before installing JFrog helm charts, you need to add the [JFrog helm repository](https://charts.jfrog.io/) to your helm client
```bash
helm repo add jfrog https://charts.jfrog.io
```

### Install Chart

#### Artifactory Connection Details
In order to connect Distribution to your Artifactory installation, you have to use a Join Key, hence it is *MANDATORY* to provide a Join Key and Jfrog Url to your Distribution installation. Here's how you do that:

Retrieve the connection details of your Artifactory installation, from the UI - https://www.jfrog.com/confluence/display/JFROG/General+Security+Settings#GeneralSecuritySettings-ViewingtheJoinKey. 

#### Initiate Installation
Provide join key and jfrog url as a parameter to the Distribution chart installation:

```bash
helm install --set distribution.joinKey=<YOUR_PREVIOUSLY_RETIREVED_JOIN_KEY> \
             --set distribution.jfrogUrl=<YOUR_PREVIOUSLY_RETIREVED_BASE_URL>  jfrog/distribution
```

### System Configuration
Distribution uses a common system configuration file - `system.yaml`. See [official documentation](https://www.jfrog.com/confluence/display/JFROG/System+YAML+Configuration+File) on its usage.

### Deploying Distribution for small/medium/large installations
In the chart directory, we have added three values files, one for each installation type - small/medium/large. These values files are recommendations for setting resources requests and limits for your installation. The values are derived from the following [documentation](https://www.jfrog.com/confluence/display/EP/Installing+on+Kubernetes#InstallingonKubernetes-Systemrequirements). You can find them in the corresponding chart directory -  values-small.yaml, values-medium.yaml and values-large.yaml

### Accessing Distribution
**NOTE:** It might take a few minutes for Distribution's public IP to become available, and the nodes to complete initial setup.
Follow the instructions outputted by the install command to get the Distribution IP and URL to access it.

### Updating Distribution
Once you have a new chart version, you can update your deployment with
```bash
helm upgrade distribution jfrog/distribution
```

If distribution was installed without providing a value to redis.password (a password was autogenerated), follow these instructions:
1. Get the current password by running:
```bash
REDIS_PASSWORD=$(kubectl get secret -n <namespace> <myrelease>-redis-secret -o jsonpath="{.data.password}" | base64 --decode)
```
2. Upgrade the release by passing the previously auto-generated secret:
```bash
helm upgrade <myrelease> jfrog/distribution --set redis.password=${REDIS_PASSWORD}
```

If distribution was installed without providing a value to postgresql.postgresqlPassword (a password was autogenerated), follow these instructions:
1. Get the current password by running:
```bash
POSTGRES_PASSWORD=$(kubectl get secret -n <namespace> <myrelease>-postgresql -o jsonpath="{.data.password}" | base64 --decode)
```
2. Upgrade the release by passing the previously auto-generated secret:
```bash
helm upgrade <myrelease> jfrog/distribution --set postgresql.postgresqlPassword=${POSTGRES_PASSWORD}
```

### Create a unique Master Key
JFrog Distribution requires a unique master key to be used by all micro-services in the same cluster. By default the chart has one set in values.yaml (`distribution.masterKey`).

**This key is for demo purpose and should not be used in a production environment!**

You should generate a unique one and pass it to the template at install/upgrade time.
```bash
# Create a key
export MASTER_KEY=$(openssl rand -hex 32)
echo ${MASTER_KEY}

# Pass the created master key to helm
helm install --set distribution.masterKey=${MASTER_KEY} -n distribution jfrog/distribution
```
**NOTE:** Make sure to pass the same master key with `--set distribution.masterKey=${MASTER_KEY}` on all future calls to `helm install` and `helm upgrade`!

### High Availability
JFrog Distribution can run in High Availability by having multiple replicas of the Distribution service.

To enable this, pass replica count to the `helm install` and `helm upgrade` commands.
```bash
# Run 3 replicas of the Distribution service
helm install --name distribution --set replicaCount=3 jfrog/distribution
```

### External Database

**For production grade installations it is recommended to use an external PostgreSQL with a static password**

There is an option to use an external PostgreSQL database for your Distribution.

To use an external **PostgreSQL**, You need to set the Distribution **PostgreSQL** connection details
```bash
export POSTGRES_URL=
export POSTGRES_USERNAME=
export POSTGRES_PASSWORD=

helm install --name distribution \
    --set database.url=${POSTGRES_URL} \
    --set database.user=${POSTGRES_USERNAME} \
    --set database.password=${POSTGRES_PASSWORD} \
    jfrog/distribution
```
**NOTE:** The Database password is saved as a Kubernetes secret


#### Use existing secrets for PostgreSQL connection details
You can use already existing secrets for managing the database connection details.

Pass them to the install command like this
```bash
export POSTGRES_USERNAME_SECRET_NAME=
export POSTGRES_USERNAME_SECRET_KEY=
export POSTGRES_PASSWORD_SECRET_NAME=
export POSTGRES_PASSWORD_SECRET_KEY=

helm install --name distribution \
    --set database.secrets.user.name=${POSTGRES_USERNAME_SECRET_NAME} \
    --set database.secrets.user.key=${POSTGRES_USERNAME_SECRET_KEY} \
    --set database.secrets.password.name=${POSTGRES_PASSWORD_SECRET_NAME} \
    --set database.secrets.password.key=${POSTGRES_PASSWORD_SECRET_KEY} \
    jfrog/distribution
```

## Upgrade
Upgrading Distribution is a simple helm command
```bash
helm upgrade distribution jfrog/distribution
```

**NOTE:** Check for any version specific upgrade nodes in [CHANGELOG.md]

### Non compatible upgrades
In cases where a new version is not compatible with existing deployed version (look in CHANGELOG.md) you should
* Deploy new version along side old version (set a new release name)
* Copy configurations and data from old deployment to new one (/var/opt/jfrog)
* Update DNS to point to new Distribution service
* Remove old release

## Logger sidecars
This chart provides the option to add sidecars to tail various logs from Distribution containers. See the available values in `values.yaml`

 Get list of containers in the pod
```bash
kubectl get pods -n <NAMESPACE> <POD_NAME> -o jsonpath='{.spec.containers[*].name}' | tr ' ' '\n'
```

 View specific log
```bash
kubectl logs -n <NAMESPACE> <POD_NAME> -c <LOG_CONTAINER_NAME>
```

## Custom init containers
There are cases where a special, unsupported init processes is needed like checking something on the file system or testing something before spinning up the main container.

For this, there is a section for writing a custom init container in the [values.yaml](values.yaml). By default it's commented out
```
distribution:
  ## Add custom init containers
  customInitContainers: |
    ## Init containers template goes here ##
```

### Custom volumes
There are cases where you'd like custom files mounted onto your container's file system.

For this, there is a section for defining custom volumes in the [vaules.yaml](values.yaml).  By default they are left empty.
You can mount custom volumes onto both the distribution and the distributor podsm like so:
```
common:
  ## Add custom volumes
  customVolumes: |
  #  - name: custom-script
  #    configMap:
  #      name: custom-script

distribution:
  ## Add custom volumeMounts
  customVolumeMounts: |
  #  - name: custom-script
  #    mountPath: "/scripts/script.sh"
  #    subPath: script.sh

distributor:
  ## Add custom volumeMounts
  customVolumeMounts: |
  #  - name: custom-script
  #    mountPath: "/scripts/script.sh"
  #    subPath: script.sh
```

## Configuration
The following table lists the configurable parameters of the distribution chart and their default values.

|         Parameter                               |           Description                                                  |               Default                                              |
|------------------------------------------------ |------------------------------------------------------------------------|--------------------------------------------------------------------|
| `imagePullSecrets`                              | Docker registry pull secret                                            |                                                                    |
| `replicaCount`                                  | HA - Number of instances per service                                   |                                                                    |
| `serviceAccount.create`                         | Specifies whether a ServiceAccount should be created                   | `true`                                                             |
| `serviceAccount.name`                           | The name of the ServiceAccount to create                               | Generated using fullname template                                  |
| `rbac.create`                                   | Specifies whether RBAC resources should be created                     | `true`                                                             |
| `rbac.role.rules`                               | Rules to create                                                        | `[]`                                                               |
| `postgresql.enabled`                            | Enable PostgreSQL                                                      | `true`                                                             |
| `postgresql.imageTag`                           | PostgreSQL image tag                                                   | `9.6.11`                                                           |
| `postgresql.postgresqlDatabase`                 | PostgreSQL database name                                               | `distribution`                                                     |
| `postgresql.postgresqlUsername`                 | PostgreSQL database username                                           | `distribution`                                                     |
| `postgresql.postgresqlPassword`                 | PostgreSQL database password                                           | ` `                                                                |
| `postgresql.postgresqlExtendedConf.listenAddresses` | PostgreSQL listen address                                          | `"'*'"`                                                            |
| `postgresql.postgresqlExtendedConf.maxConnections`  | PostgreSQL max_connections parameter                               | `1500`                                                             |
| `postgresql.service.port`                       | PostgreSQL service port                                                | `5432`                                                             |
| `postgresql.persistence.enabled`                | PostgreSQL persistence enabled                                         | `true`                                                             |
| `postgresql.persistence.size`                   | PostgreSQL persistent disk size                                        | `50Gi`                                                             |
| `postgresql.persistence.existingClaim`          | PostgreSQL persistence use existing PVC                                | ` `                                                                |
| `postgresql.resources.requests.memory`          | PostgreSQL initial memory request                                      | ` `                                                                |
| `postgresql.resources.requests.cpu`             | PostgreSQL initial cpu request                                         | ` `                                                                |
| `postgresql.resources.limits.memory`            | PostgreSQL memory limit                                                | ` `                                                                |
| `postgresql.resources.limits.cpu`               | PostgreSQL cpu limit                                                   | ` `                                                                |
| `postgresql.nodeSelector`                       | PostgreSQL node selector                                               | `{}`                                                               |
| `postgresql.affinity`                           | PostgreSQL node affinity                                               | `{}`                                                               |
| `postgresql.tolerations`                        | PostgreSQL node tolerations                                            | `[]`                                                               |
| `redis.password`                                | Redis password                                                         | ` `                                                                |
| `redis.port`                                    | Redis Port                                                             | `6379`                                                             |
| `redis.persistence.enabled`                     | Redis use a PVC to persist data                                        | `true`                                                             |
| `redis.persistence.existingClaim`               | Redis use an existing PVC to persist data                              | `nil`                                                              |
| `redis.persistence.storageClass`                | Redis storage class of backing PVC                                     | `generic`                                                          |
| `redis.persistence.size`                        | Redis size of data volume                                              | `10Gi`                                                             |
| `redis.nodeSelector`                            | Redis node selector                                                    | `{}`                                                               |
| `redis.affinity`                                | Redis node affinity                                                    | `{}`                                                               |
| `redis.tolerations`                             | Redis node tolerations                                                 | `[]`                                                               |
| `logger.image.repository`                       | Repository for logger image                                            | `busybox`                                                          |
| `logger.image.tag`                              | Tag for logger image                                                   | `1.30`                                                             |
| `common.uid`                                    | Distribution and Distributor process user ID                           | `1020`                                                             |
| `common.gid`                                    | Distribution and Distributor process group ID                          | `1020`                                                             |
| `common.customVolumes`                          | Custom Volumes for Distribution                                        | see [values.yaml](values.yaml)                                     |
| `distribution.name`                             | Distribution name                                                      | `distribution`                                                     |
| `distribution.image.pullPolicy`                 | Container pull policy                                                  | `IfNotPresent`                                                     |
| `distribution.image.repository`                 | Container image                                                        | `docker.bintray.io/jf-distribution`                                  |
| `distribution.image.version`                    | Container image tag                                                    | `.Chart.AppVersion`                                                |
| `distribution.service.type`                     | Distribution service type                                              | `ClusterIP`                                                     |
| `distribution.customInitContainers`             | Custom init containers for Distribution                                |                                                                    |
| `distribution.customVolumeMounts`               | Custom Volume Mounts for Distribution                                  | see [values.yaml](values.yaml)                                     |
| `distribution.externalPort`                     | Distribution service external port                                     | `80`                                                               |
| `distribution.internalPort`                     | Distribution service internal port                                     | `8080`                                                             |
| `distribution.masterKey`                        | Distribution Master Key (can be generated with `openssl rand -hex 32`) | `BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB` |
| `distribution.jfrogUrl`                         | Main Artifactory URL, without the `/artifactory` prefix . Mandatory    | ` `                                                                |
| `database.type`                                 | External database type (`postgresql`)                                  | `postgresql`                                      |
| `database.driver`                               | External database driver                                               | 
`org.postgresql.Driver`                           |
| `database.url`                                  | External database url                                                  |
` `                                               |
| `database.user`                                 | External database user                                                 |
` `                                               |
| `database.password`                             | External database password                                             |
` `                                               |
| `distribution.joinKey`                          | Distribution Join Key . Mandatory                                      | ` `                                                                |
| `distribution.serviceId`                        | Distribution service ID                                                | ` `                                                                |
| `distribution.env.artifactoryUrl`               | Distribution Environment Artifactory URL                               | ` `                                                                |
| `distribution.persistence.mountPath`            | Distribution persistence volume mount path                             | `"/jf-distribution"`                                               |
| `distribution.persistence.enabled`              | Distribution persistence volume enabled                                | `true`                                                             |
| `distribution.persistence.storageClass`         | Storage class of backing PVC                                           | `nil`                                                              |
| `distribution.persistence.existingClaim`        | Provide an existing PersistentVolumeClaim                              | `nil`                                                              |
| `distribution.persistence.accessMode`           | Distribution persistence volume access mode                            | `ReadWriteOnce`                                                    |
| `distribution.persistence.size`                 | Distribution persistence volume size                                   | `50Gi`                                                             |
| `distribution.preStartCommand`                  | Distribution Command to run before the startup                         |                                                                    |
| `distribution.nodeSelector`                     | Distribution node selector                                             | `{}`                                                               |
| `distribution.affinity`                         | Distribution node affinity                                             | `{}`                                                               |
| `distribution.tolerations`                      | Distribution node tolerations                                          | `[]`                                                               |
| `distribution.loggers`                          | Distribution loggers (see values.yaml for possible values)             | ` `                                                                |
| `distribution.loggersResources.requests.memory` | Distribution loggers initial memory request                            |                                                                    |
| `distribution.loggersResources.requests.cpu`    | Distribution loggers initial cpu request                               |                                                                    |
| `distribution.loggersResources.limits.memory`   | Distribution loggers memory limit                                      |                                                                    |
| `distribution.loggersResources.limits.cpu`      | Distribution loggers cpu limit                                         |                                                                    |
| `distribution.systemYaml`                       | Distribution system configuration (`system.yaml`)                      | `see values.yaml`                                                  |
| `distributor.name`                              | Distribution name                                                      | `distribution`                                                     |
| `distributor.image.pullPolicy`                  | Container pull policy                                                  | `IfNotPresent`                                                     |
| `distributor.image.repository`                  | Container image                                                        | `docker.bintray.io/jf-distribution`                                  |
| `distributor.image.version`                     | Container image tag                                                    | `.Chart.AppVersion`                                                |
| `distributor.persistence.mountPath`             | Distributor persistence volume mount path                              | `"/bt-distributor"`                                                |
| `distributor.persistence.existingClaim`         | Provide an existing PersistentVolumeClaim                              | `nil`                                                              |
| `distributor.persistence.storageClass`          | Storage class of backing PVC                                           | `nil (uses alpha storage class annotation)`                        |
| `distributor.persistence.enabled`               | Distributor persistence volume enabled                                 | `true`                                                             |
| `distributor.persistence.accessMode`            | Distributor persistence volume access mode                             | `ReadWriteOnce`                                                    |
| `distributor.persistence.size`                  | Distributor persistence volume size                                    | `50Gi`                                                             |
| `distributor.customVolumeMounts`                | Distributor Custom Volume Mounts                                       | see [values.yaml](values.yaml)                                     |
| `distributor.preStartCommand`                   | Distributor Command to run before the startup                          |                                                                    |
| `distributor.nodeSelector`                      | Distributor node selector                                              | `{}`                                                               |
| `distributor.affinity`                          | Distributor node affinity                                              | `{}`                                                               |
| `distributor.tolerations`                       | Distributor node tolerations                                           | `[]`                                                               |
| `distributor.loggers`                           | Distributor loggers (see values.yaml for possible values)              | ` `                                                                |
| `distributor.loggersResources.requests.memory`  | Distributor loggers initial memory request                             |                                                                    |
| `distributor.loggersResources.requests.cpu`     | Distributor loggers initial cpu request                                |                                                                    |
| `distributor.loggersResources.limits.memory`    | Distributor loggers memory limit                                       |                                                                    |
| `distributor.loggersResources.limits.cpu`       | Distributor loggers cpu limit                                          |                                                                    |
| `router.name`                                   | Router name                                                            | `router`                                                           |
| `router.image.pullPolicy`                       | Container pull policy                                                  | `IfNotPresent`                                                     |
| `router.image.repository`                       | Container image                                                        | `docker.bintray.io/jfrog/router`                                     |
| `router.image.version`                          | Container image tag                                                    | `.Chart.AppVersion`                                                |
      
Specify each parameter using the `--set key=value[,key=value]` argument to `helm install`.

## Useful links
- https://www.jfrog.com/confluence/display/EP/Getting+Started
- https://www.jfrog.com/confluence/display/DIST/Installing+Distribution
- https://www.jfrog.com/confluence/
